@@ -6,10 +6,11 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 01:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 import base64
+import json
 from typing import Optional
 
 from PySide6.QtCore import QObject, Signal, Slot, QRunnable
@@ -39,9 +40,12 @@ class StreamWorker(QObject, QRunnable):
         error = None
         fn_args_buffers = {}
         citations = []
+        files = []
         img_path = self.window.core.image.gen_unique_path(self.ctx)
         is_image = False
         is_code = False
+        force_func_call = False
+        stopped = False
 
         data = {
             "meta": self.ctx.meta,
@@ -56,8 +60,29 @@ class StreamWorker(QObject, QRunnable):
                 for chunk in self.ctx.stream:
                     # if force stop then break
                     if self.window.controller.kernel.stopped():
-                        break
+                        # save current context
+                        if tool_calls:
+                            self.ctx.force_call = force_func_call
+                            self.window.core.debug.info("[chat] Tool calls found, unpacking...")
+                            self.window.core.command.unpack_tool_calls_chunks(self.ctx, tool_calls, append_output=True)
+                        # append images
+                        if is_image:
+                            self.window.core.debug.info("[chat] Image generation call found")
+                            self.ctx.images = [img_path]  # save image path to ctx
+                        self.window.core.ctx.update_item(self.ctx)  # update ctx
+                        stopped = True
                     if error is not None:
+                        # save current context
+                        if tool_calls:
+                            self.ctx.force_call = force_func_call
+                            self.window.core.debug.info("[chat] Tool calls found, unpacking...")
+                            self.window.core.command.unpack_tool_calls_chunks(self.ctx, tool_calls, append_output=True)
+                        # append images
+                        if is_image:
+                            self.window.core.debug.info("[chat] Image generation call found")
+                            self.ctx.images = [img_path]  # save image path to ctx
+                        self.window.core.ctx.update_item(self.ctx)  # update ctx
+                        stopped = True
                         break  # break if error
 
                     etype = None
@@ -124,7 +149,50 @@ class StreamWorker(QObject, QRunnable):
 
                     # OpenAI Responses API
                     elif chunk_type == "api_chat_responses":
-                        if etype == "response.output_text.delta":
+                        if etype == "response.completed":
+                            # MCP tools
+                            for item in chunk.response.output:
+                                # MCP: list tools
+                                if item.type == "mcp_list_tools":
+                                    tools = item.tools
+                                    self.window.core.gpt.responses.mcp_tools = tools  # store MCP tools for later use
+                                # MCP: tool call
+                                elif item.type == "mcp_call":
+                                    call = {
+                                        "id": item.id,
+                                        "type": "mcp_call",
+                                        "approval_request_id": item.approval_request_id,
+                                        "arguments": item.arguments,
+                                        "error": item.error,
+                                        "name": item.name,
+                                        "output": item.output,
+                                        "server_label": item.server_label,
+                                    }
+                                    tool_calls.append({
+                                        "id": item.id,
+                                        "call_id": "",
+                                        "type": "function",
+                                        "function": {
+                                            "name": item.name,
+                                            "arguments": item.arguments
+                                        }
+                                    })
+                                    self.ctx.extra["mcp_call"] = call
+                                    self.window.core.ctx.update_item(self.ctx)
+                                # MCP: approval request
+                                elif item.type == "mcp_approval_request":
+                                    call = {
+                                        "id": item.id,
+                                        "type": "mcp_call",
+                                        "arguments": item.arguments,
+                                        "name": item.name,
+                                        "server_label": item.server_label,
+                                    }
+                                    self.ctx.extra["mcp_approval_request"] = call
+                                    self.window.core.ctx.update_item(self.ctx)
+
+                        # text chunk
+                        elif etype == "response.output_text.delta":
                             response = chunk.delta
 
                         # ---------- function_call ----------
@@ -156,6 +224,23 @@ class StreamWorker(QObject, QRunnable):
                                 url_citation = chunk.annotation['url']
                                 citations.append(url_citation)
                                 self.ctx.urls = citations
+                            if chunk.annotation['type'] == "container_file_citation":
+                                container_id = chunk.annotation['container_id']
+                                file_id = chunk.annotation['file_id']
+                                files.append({
+                                    "container_id": container_id,
+                                    "file_id": file_id,
+                                })
+
+                        # ---------- computer use ----------
+                        elif etype == "response.reasoning_summary_text.delta":
+                            response = chunk.delta
+
+                        elif etype == "response.output_item.done":
+                           tool_calls, has_calls = self.window.core.gpt.computer.handle_stream_chunk(self.ctx, chunk, tool_calls)
+                           if has_calls:
+                               force_func_call = True  # force function call if computer use found
+
 
                         # ---------- code interpreter ----------
                         elif etype == "response.code_interpreter_call_code.delta":
@@ -178,6 +263,7 @@ class StreamWorker(QObject, QRunnable):
                         # ---------- response ID ----------
                         elif etype == "response.created":
                             self.ctx.msg_id = chunk.response.id
+                            self.window.core.ctx.update_item(self.ctx)  # prevent non-existing response ID
 
                         # ---------- end / error ----------
                         elif etype in {"response.done", "response.failed", "error"}:
@@ -234,7 +320,7 @@ class StreamWorker(QObject, QRunnable):
                         if chunk is not None:
                             response = str(chunk)
 
-                    if response is not None and response != "":
+                    if response is not None and response != "" and not stopped:
                         if begin and response == "":  # prevent empty beginning
                             continue
                         output += response
@@ -251,6 +337,7 @@ class StreamWorker(QObject, QRunnable):
 
                 # unpack and store tool calls
                 if tool_calls:
+                    self.ctx.force_call = force_func_call
                     self.window.core.debug.info("[chat] Tool calls found, unpacking...")
                     self.window.core.command.unpack_tool_calls_chunks(self.ctx, tool_calls)
 
@@ -265,6 +352,18 @@ class StreamWorker(QObject, QRunnable):
         # update ctx
         self.ctx.output = output
         self.ctx.set_tokens(self.ctx.input_tokens, output_tokens)
+        self.window.core.ctx.update_item(self.ctx)  # update ctx
+
+        if stopped:
+            return
+
+        # if files from container are found, download them and append to ctx
+        if files:
+            self.window.core.debug.info("[chat] Container files found, downloading...")
+            try:
+                self.window.core.gpt.container.download_files(self.ctx, files)
+            except Exception as e:
+                self.window.core.debug.error(f"[chat] Error downloading container files: {e}")
 
         self.end.emit(self.ctx)
         if error:
