@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.01 01:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 from typing import Optional
@@ -17,6 +17,7 @@ from pygpt_net.core.types import (
     MODE_AUDIO,
     MODE_ASSISTANT,
     MODE_LLAMA_INDEX,
+    MODE_COMPUTER,
 )
 from pygpt_net.core.events import Event, AppEvent, KernelEvent, RenderEvent
 from pygpt_net.core.bridge.context import BridgeContext, MultimodalContext
@@ -118,9 +119,13 @@ class Text:
             ctx.sub_reply = True  # mark as sub reply
             ctx.input_name = prev_ctx.input_name
             ctx.output_name = prev_ctx.output_name
+            ctx.extra["sub_reply"] = True  # mark as sub reply in extra data
         else:
             self.window.core.ctx.set_last_item(ctx)  # store last item
 
+        if reply:
+            ctx.extra["sub_reply"] = True  # mark as sub reply in extra data
+
         self.window.controller.files.uploaded_ids = []  # clear uploaded files ids at the beginning
 
         # assistant: create thread, upload attachments
@@ -153,14 +158,6 @@ class Text:
         self.window.dispatch(event)
         sys_prompt = event.data['value']
 
-        # tool calls
-        disable_native_func_calls = False
-        if mode == MODE_LLAMA_INDEX:
-            # check if index is selected
-            if self.window.controller.idx.index_selected():
-                disable_native_func_calls = True
-                # ^^^ native func calls allowed only for LLM call, not for the query engine
-
         # build final prompt (+plugins)
         sys_prompt = self.window.core.prompt.prepare_sys_prompt(
             mode=mode,
@@ -169,7 +166,6 @@ class Text:
             ctx=ctx,
             reply=reply,
             internal=internal,
-            disable_native_tool_calls=disable_native_func_calls,
         )
 
         self.window.controller.chat.log("Appending input to chat window...")
