@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.06.28 16:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 import datetime
@@ -23,9 +23,10 @@ from pygpt_net.core.types import (
     MODE_EXPERT,
     MODE_LANGCHAIN,
     MODE_LLAMA_INDEX,
-    MODE_VISION, 
+    MODE_VISION,
     MODE_IMAGE,
     MODE_RESEARCH,
+    MODE_COMPUTER,
 )
 from pygpt_net.item.preset import PresetItem
 from pygpt_net.utils import trans
@@ -106,6 +107,10 @@ class Editor:
                 "type": "bool",
                 "label": "preset.llama_index",
             },
+            MODE_COMPUTER: {
+                "type": "bool",
+                "label": "preset.computer",
+            },
             "model": {
                 "label": "toolbox.model.label",
                 "type": "combo",
@@ -308,6 +313,8 @@ class Editor:
                 data.audio = True
             elif mode == MODE_RESEARCH:
                 data.research = True
+            elif mode == MODE_COMPUTER:
+                data.computer = True
 
         options = {}
         data_dict = data.to_dict()
@@ -377,6 +384,7 @@ class Editor:
             MODE_EXPERT,
             MODE_AGENT_LLAMA,
             MODE_AUDIO,
+            MODE_COMPUTER,
         ]
 
         # disallow editing default preset
@@ -407,7 +415,9 @@ class Editor:
 
         # validate filename
         id = self.window.controller.presets.validate_filename(id)
+        is_new = False
         if id not in self.window.core.presets.items:
+            is_new = True
             self.window.core.presets.items[id] = self.window.core.presets.build()
         elif not force:
             self.window.ui.dialogs.confirm(
@@ -450,8 +460,11 @@ class Editor:
         self.current = self.window.core.presets.items[id].uuid
 
         # save
+        no_scroll = False
+        if not is_new:
+            no_scroll = True
         self.window.core.presets.save(id)
-        self.window.controller.presets.refresh()
+        self.window.controller.presets.refresh(no_scroll=no_scroll)
 
         # close dialog
         if close:
@@ -469,11 +482,15 @@ class Editor:
         # sort by name
         self.window.core.presets.sort_by_name()
 
-        # switch to editing preset
-        self.window.controller.presets.set(mode, id)
+        # switch to editing preset, if new
+        if is_new:
+            self.window.controller.presets.set(mode, id)
 
         # update presets list
-        self.window.controller.presets.refresh()
+        no_scroll = False
+        if not is_new:
+            no_scroll = True
+        self.window.controller.presets.refresh(no_scroll=no_scroll)
 
     def assign_data(self, id: str):
         """
