@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.07.19 17:00:00                  #
 # ================================================== #
 
 import re
@@ -37,6 +37,7 @@ class Presets:
         self.window = window
         self.editor = Editor(window)
         self.locked = False
+        self.selected = []
 
     def setup(self):
         """Setup presets"""
@@ -58,6 +59,7 @@ class Presets:
         self.window.controller.ui.update()
         self.window.controller.model.select_current()
         self.window.dispatch(AppEvent(AppEvent.PRESET_SELECTED))  # app event
+        self.set_selected(idx)
 
     def get_current(self) -> Optional[PresetItem]:
         """
@@ -280,15 +282,23 @@ class Presets:
         # select model
         self.select_model()
 
-    def select_current(self):
-        """Select preset by current"""
+    def select_current(self, no_scroll: bool = False):
+        """
+        Select preset by current
+
+        :param no_scroll: do not scroll to current preset
+        """
         mode = self.window.core.config.get('mode')
         preset_id = self.window.core.config.get('preset')
         items = self.window.core.presets.get_by_mode(mode)
         if preset_id in items:
             idx = list(items.keys()).index(preset_id)
-            current = self.window.ui.models['preset.presets'].index(idx, 0)
-            self.window.ui.nodes['preset.presets'].setCurrentIndex(current)
+            # current = self.window.ui.models['preset.presets'].index(idx, 0)
+            if no_scroll:
+                self.window.ui.nodes['preset.presets'].store_scroll_position()
+            self.window.ui.nodes['preset.presets'].select_by_idx(idx)
+            if no_scroll:
+                self.window.ui.nodes['preset.presets'].restore_scroll_position()
 
     def select_default(self):
         """Set default preset"""
@@ -403,24 +413,34 @@ class Presets:
                             self.window.controller.model.init_list()
                             self.window.controller.model.select_current()
 
-    def refresh(self):
-        """Refresh presets"""
+    def refresh(self, no_scroll: bool = False):
+        """
+        Refresh presets
+
+        :param no_scroll: do not scroll to current
+        """
         mode = self.window.core.config.get('mode')
         if mode == MODE_ASSISTANT:
             return
 
+        if no_scroll:
+            self.window.ui.nodes['preset.presets'].store_scroll_position()
         self.select_default()  # if no preset then select previous from current presets
         self.update_current()  # apply preset data to current config
         self.update_data()  # update config and prompt from preset data
         self.window.controller.mode.update_temperature()
         self.update_list()  # update presets list only
         self.select_current()  # select current preset on list
+        if no_scroll:
+            self.window.ui.nodes['preset.presets'].restore_scroll_position()
+
 
     def update_list(self):
         """Update presets list"""
         mode = self.window.core.config.get('mode')
         items = self.window.core.presets.get_by_mode(mode)
         self.window.ui.toolbox.presets.update_presets(items)
+        self.clear_selected()
 
     def reset(self):
         """Reset preset data"""
@@ -451,12 +471,15 @@ class Presets:
             if preset is not None and preset != "":
                 if preset in self.window.core.presets.items:
                     new_id = self.window.core.presets.duplicate(preset)
-                    self.window.core.config.set('preset', new_id)
-                    self.refresh()
+                    #self.window.core.config.set('preset', new_id)
+                    self.update_list()
+                    self.refresh(no_scroll=True)
                     idx = self.window.core.presets.get_idx_by_id(mode, new_id)
                     self.editor.edit(idx)
                     self.window.update_status(trans('status.preset.duplicated'))
 
+
+
     def enable(self, idx: Optional[int] = None):
         """
         Enable preset
@@ -550,7 +573,7 @@ class Presets:
                         self.window.core.config.set('preset', None)
                         self.window.ui.nodes['preset.prompt'].setPlainText("")
                     self.window.core.presets.remove(preset_id, True)
-                    self.refresh()
+                    self.refresh(no_scroll=True)
                     self.window.update_status(trans('status.preset.deleted'))
 
     def restore(self, force: bool = False):
@@ -618,3 +641,33 @@ class Presets:
         self.window.core.presets.load()
         self.refresh()
         self.locked = False
+
+    def add_selected(self, id: int):
+        """
+        Add selection ID to selected list
+
+        :param id: preset ID
+        """
+        if id not in self.selected:
+            self.selected.append(id)
+
+    def remove_selected(self, id: int):
+        """
+        Remove selection ID from selected list
+
+        :param id: preset ID
+        """
+        if id in self.selected:
+            self.selected.remove(id)
+
+    def set_selected(self, id: int):
+        """
+        Set selected ID in selected list
+
+        :param id: preset ID
+        """
+        self.selected = [id] if id is not None else []
+
+    def clear_selected(self):
+        """Clear selected list"""
+        self.selected = []
