@@ -6,11 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 15:00:00                  #
+# Updated Date: 2025.07.20 23:00:00                  #
 # ================================================== #
 
 from typing import Any, Optional
 
+from PySide6.QtCore import QTimer
+
 from pygpt_net.core.events import AppEvent, RenderEvent
 from pygpt_net.core.tabs.tab import Tab
 from pygpt_net.item.ctx import CtxMeta
@@ -18,6 +20,9 @@ from pygpt_net.utils import trans
 
 
 class Tabs:
+
+    TAB_CHAT_MAX_CHARS = 15  # max chars for chat tab title
+
     def __init__(self, window=None):
         """
         UI tabs controller
@@ -33,6 +38,7 @@ class Tabs:
         self.column_idx = 0
         self.tmp_column_idx = 0
         self.locked = False
+        self.create_new_on_tab = True
         self.col = {}
 
     def setup(self):
@@ -49,6 +55,11 @@ class Tabs:
         if not state:
             self.window.ui.splitters['columns'].setSizes([1, 0])
 
+    def debug(self):
+        """Debug tabs if enabled"""
+        if self.window.controller.dialogs.debug.is_active("tabs"):
+            self.window.core.tabs.toggle_debug(True)
+
     def add(
             self,
             type: int,
@@ -101,10 +112,12 @@ class Tabs:
             tool_id=tool_id
         )
         self.switch_tab_by_idx(tab.idx, column_idx)  # switch to new tab
+        self.debug()
 
     def reload_titles(self):
         """Reload tab titles"""
         self.window.core.tabs.reload_titles()
+        self.debug()
 
     def update_current(self):
         """Update current tab"""
@@ -114,12 +127,14 @@ class Tabs:
             self.col[curr_column] = -1
         if curr_tab is not None:
             self.col[curr_column] = curr_tab.pid
+        self.debug()
 
     def reload(self):
         """Reload tabs"""
         self.window.core.tabs.reload()
         event = RenderEvent(RenderEvent.PREPARE)
         self.window.dispatch(event)
+        self.debug()
 
     def reload_after(self):
         """Reload tabs after"""
@@ -131,6 +146,7 @@ class Tabs:
                 self.window.ui.nodes['output_plain'][pid].setVisible(False)
                 self.window.ui.nodes['output'][pid].setVisible(True)
         #self.switch_tab(Tab.TAB_CHAT)
+        self.debug()
 
     def on_tab_changed(
             self,
@@ -152,9 +168,11 @@ class Tabs:
             self.appended = False
             if tab.type == Tab.TAB_CHAT:
                 self.current = idx
-                meta = self.window.controller.ctx.new()  # new context
-                if meta is not None:
-                    self.window.controller.ctx.load(meta.id)  # reload
+                if self.create_new_on_tab:
+                    meta = self.window.controller.ctx.new()  # new context
+                    if meta is not None:
+                        self.window.controller.ctx.load(meta.id)  # reload
+                self.create_new_on_tab = True
 
         prev_tab = self.current
         prev_column = self.column_idx
@@ -169,7 +187,9 @@ class Tabs:
             self.window.controller.notepad.on_open(idx, column_idx)
         elif tab.type == Tab.TAB_CHAT:
             # get meta for selected tab, if not loaded yet then append meta here
-            meta_id = self.window.core.ctx.output.prepare_meta(tab)
+            meta_id = tab.data_id
+            if meta_id is None:
+                meta_id = self.window.core.ctx.output.prepare_meta(tab)
             meta = self.window.core.ctx.get_meta_by_id(meta_id)
             if meta is not None:
                 self.window.controller.ctx.load(meta.id)  # reload renderer
@@ -184,9 +204,9 @@ class Tabs:
             self.window.dispatch(AppEvent(AppEvent.TAB_SELECTED))  # app event
 
         self.on_changed()
-
         self.window.controller.ui.update()
         self.update_current()
+        self.debug()
 
     def on_changed(self):
         """On Tab or column changed event (any)"""
@@ -194,6 +214,7 @@ class Tabs:
         if tab is None:
             return
         self.window.controller.audio.on_tab_changed(tab)
+        self.debug()
 
     def get_current_idx(self, column_idx: int = 0) -> int:
         """
@@ -287,6 +308,7 @@ class Tabs:
                 self.window.controller.ctx.select_on_list_only(tab.data_id)
         self.window.controller.ui.update()
         self.update_current()
+        self.debug()
 
     def on_tab_clicked(
             self,
@@ -304,6 +326,7 @@ class Tabs:
         self.on_column_changed()
         self.on_changed()
         self.update_current()
+        self.debug()
 
     def on_column_focus(self, idx: int):
         """
@@ -317,6 +340,7 @@ class Tabs:
         self.on_column_changed()
         self.on_changed()
         self.update_current()
+        self.debug()
 
     def on_tab_dbl_clicked(
             self,
@@ -332,6 +356,7 @@ class Tabs:
         self.column_idx = column_idx
         self.on_tab_changed(idx, column_idx)
         self.update_current()
+        self.debug()
 
     def on_tab_closed(
             self,
@@ -349,6 +374,7 @@ class Tabs:
         self.window.core.tabs.remove_tab_by_idx(idx, column_idx)
         self.on_changed()
         self.update_current()
+        self.debug()
 
     def on_tab_moved(
             self,
@@ -365,6 +391,7 @@ class Tabs:
             return
         self.window.core.tabs.update()
         self.update_current()
+        self.debug()
 
     def close(
             self,
@@ -379,6 +406,7 @@ class Tabs:
         """
         self.on_tab_closed(idx, column_idx)
         self.update_current()
+        self.debug()
 
     def close_all(
             self,
@@ -405,6 +433,7 @@ class Tabs:
         self.window.core.tabs.remove_all_by_type(type, column_idx)
         self.on_changed()
         self.update_current()
+        self.debug()
 
     def next_tab(self):
         """Switch to next tab"""
@@ -492,6 +521,7 @@ class Tabs:
         """
         tabs = self.window.ui.layout.get_active_tabs()
         tabs.setTabToolTip(self.current, tooltip)
+        self.debug()
 
     def rename(
             self,
@@ -530,6 +560,7 @@ class Tabs:
         self.window.core.tabs.update_title(idx, name)
         if close:
             self.window.ui.dialog['rename'].close()
+        self.debug()
 
     def update_current_name(self, name: str):
         """
@@ -556,9 +587,10 @@ class Tabs:
         tabs = self.window.ui.layout.get_active_tabs()
         tooltip = title
         tabs.setTabToolTip(idx, tooltip)
-        if len(title) > 8:
-            title = title[:8] + '...'  # truncate to max 8 chars
+        if len(title) > self.TAB_CHAT_MAX_CHARS:
+            title = title[:self.TAB_CHAT_MAX_CHARS] + '...'  # truncate to max 8 chars
         self.window.core.tabs.update_title(idx, title, tooltip)
+        self.debug()
 
     def update_title_current(self, title: str):
         """
@@ -579,6 +611,7 @@ class Tabs:
         if tab is not None and tab.type == Tab.TAB_CHAT:
             tab.data_id = meta.id
         self.update_title_current(meta.name)
+        self.debug()
 
     def open_by_type(self, type: int):
         """
@@ -620,6 +653,7 @@ class Tabs:
         # set default column to 0
         self.column_idx = 0
         self.on_column_changed()
+        self.debug()
 
     def move_tab(
             self,
@@ -643,6 +677,7 @@ class Tabs:
         self.on_column_changed()
         # switch to new tab
         self.switch_tab_by_idx(tab.idx, new_column_idx)
+        self.debug()
 
     def is_current_by_type(self, type: int) -> bool:
         """
@@ -669,6 +704,59 @@ class Tabs:
             tab = self.window.core.tabs.get_tab_by_pid(pid)
             if tab is not None and tab.tool_id == tool_id:
                 return True
+        return False
+
+    def get_current_by_column(self, column_idx: int) -> Optional[Tab]:
+        """
+        Get current tab by column index
+
+        :param column_idx: column index
+        :return: current tab in given column or None if not found
+        """
+        tabs = self.window.ui.layout.get_tabs_by_idx(column_idx)
+        if tabs is None:
+            return None
+        idx = tabs.currentIndex()
+        return self.window.core.tabs.get_tab_by_index(idx, column_idx)
+
+    def is_tool(self, tool_id: str) -> bool:
+        """
+        Check if one of any tabs is of given tool ID
+
+        :param tool_id: tool ID
+        :return: True if one of tab is of given tool ID
+        """
+        for col in self.col:
+            tabs = self.window.ui.layout.get_tabs_by_idx(col)
+            for i in range(tabs.count()):
+                tab = self.window.core.tabs.get_tab_by_index(i, col)
+                if tab is not None and tab.tool_id == tool_id:
+                    return True
+        return False
+
+    def get_first_tab_by_tool(self, tool_id: str) -> Tab:
+        """
+        Get first tab index by tool ID
+
+        :param tool_id: tool ID
+        :return: tab index if one of tab is of given tool ID, None otherwise
+        """
+        for col in self.col:
+            tabs = self.window.ui.layout.get_tabs_by_idx(col)
+            for i in range(tabs.count()):
+                tab = self.window.core.tabs.get_tab_by_index(i, col)
+                if tab is not None and tab.tool_id == tool_id:
+                    return tab
+
+    def switch_to_first_tab_by_tool(self, tool_id: str):
+        """
+        Switch to first tab by tool ID
+
+        :param tool_id: tool ID
+        """
+        tab = self.get_first_tab_by_tool(tool_id)
+        if tab is not None:
+            self.switch_tab_by_idx(tab.idx, tab.column_idx)
 
     def get_tool_column(self, tool_id: str) -> int:
         """
@@ -698,6 +786,68 @@ class Tabs:
                 self.switch_tab_by_idx(tab.idx, col)
                 return
 
+    def focus_by_type(
+            self,
+            type: str,
+            data_id: Optional[int] = None,
+            title: Optional[str] = None,
+            meta: Optional[CtxMeta] = None
+    ):
+        """
+        Focus by type and optionally update tab data ID and name
+
+        :param type: tab type
+        :param data_id: data ID (optional, for chat tab)
+        :param title: new tab name (optional, for chat tab)
+        :param meta: context meta (optional, for chat tab)
+        """
+        # first try to focus current tab
+        if self.get_current_type() != type:
+            # first, check in second column
+            second_column_idx = 1 if self.column_idx == 0 else 0
+            # get current tab from second column
+            tabs = self.window.ui.layout.get_tabs_by_idx(second_column_idx)
+            second_tabs_idx = tabs.currentIndex()
+            second_tab = self.window.core.tabs.get_tab_by_index(second_tabs_idx, second_column_idx)
+            if second_tab is not None and second_tab.type == type:
+                # switch to second column
+                self.on_column_focus(second_column_idx)
+                tabs.setCurrentIndex(second_tabs_idx)
+                if meta:
+                    QTimer.singleShot(100, lambda: self.window.controller.ctx.load(meta.id))
+                self.debug()
+                return
+
+            idx, column_idx, exists = self.window.core.tabs.get_min_idx_by_type_exists(type)
+            if exists:
+                tabs = self.window.ui.layout.get_tabs_by_idx(column_idx)
+                if tabs and idx:
+                    tabs.setCurrentIndex(idx)
+                    self.debug()
+                    return
+            else:
+                # if current is not type, find first tab
+                tab = self.window.core.tabs.get_first_by_type(type)
+                if tab:
+                    tabs = self.window.ui.layout.get_tabs_by_idx(tab.column_idx)
+                    if tabs:
+                        idx = tab.idx
+                        if data_id is not None:
+                            tab.data_id = data_id
+                            if title is not None:
+                                self.update_title_current(title)
+                        else:
+                            self.on_column_focus(tab.column_idx)
+                        if meta is not None:
+                            self.on_column_focus(tab.column_idx)
+                            self.window.controller.ctx.load(meta.id)
+                            QTimer.singleShot(100, lambda: self.window.controller.ctx.load(meta.id))
+                            self.on_column_focus(tab.column_idx)
+                        tabs.setCurrentIndex(idx)
+
+        self.debug()
+
+
     def on_split_screen_changed(self, state: bool):
         """
         On split screen mode changed
@@ -711,14 +861,22 @@ class Tabs:
                 self.window.ui.nodes['layout.split'].box.setChecked(state)
             self.window.core.config.save()
 
-    def enable_split_screen(self):
+    def enable_split_screen(self, update_switch: bool = False):
         """
         Enable split screen mode
+
+        :param update_switch: True if switch should be updated
         """
+        if self.window.core.config.get("layout.split", False):
+            return
+
         self.window.ui.splitters['columns'].setSizes([1, 1])
         self.window.core.config.set("layout.split", True)
         self.window.core.config.save()
 
+        if update_switch:
+            self.window.ui.nodes['layout.split'].box.setChecked(True)
+
     def disable_split_screen(self):
         """
         Disable split screen mode
