@@ -6,18 +6,22 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 01:00:00                  #
+# Updated Date: 2025.07.23 15:00:00                  #
 # ================================================== #
 
 import asyncio
-from typing import Optional, Dict, Any, List
+import copy
+import re
+from typing import Optional, Dict, Any, List, Tuple, Union
 
+from llama_index.core.base.llms.types import ChatMessage, MessageRole
 from llama_index.core.tools import FunctionTool
 from llama_index.core.workflow import Context
 from llama_index.core.agent.workflow import (
     ToolCall,
     ToolCallResult,
     AgentStream,
+    AgentOutput,
 )
 
 from pygpt_net.core.bridge.context import BridgeContext
@@ -28,7 +32,7 @@ from pygpt_net.core.types import (
     AGENT_MODE_STEP,
     AGENT_MODE_WORKFLOW,
 )
-from pygpt_net.core.events import Event, KernelEvent
+from pygpt_net.core.events import Event, KernelEvent, RenderEvent
 from pygpt_net.item.ctx import CtxItem
 from pygpt_net.utils import trans
 
@@ -79,9 +83,9 @@ class Runner:
             self.window.core.agents.tools.agent_idx = agent_idx
             system_prompt = context.system_prompt
             max_steps = self.window.core.config.get("agent.llama.steps", 10)
-            tools = self.window.core.agents.tools.prepare(context, extra)
-            plugin_tools = self.window.core.agents.tools.get_plugin_tools(context, extra)
-            plugin_specs = self.window.core.agents.tools.get_plugin_specs(context, extra)
+            tools = self.window.core.agents.tools.prepare(context, extra, force=True)
+            plugin_tools = self.window.core.agents.tools.get_plugin_tools(context, extra, force=True)
+            plugin_specs = self.window.core.agents.tools.get_plugin_specs(context, extra, force=True)
             retriever_tool = self.window.core.agents.tools.get_retriever_tool(context, extra)
             history = self.window.core.agents.memory.prepare(context)
             llm = self.window.core.idx.llm.get(model, stream=False)
@@ -89,6 +93,24 @@ class Runner:
             if self.window.core.plugins.get_option("cmd_code_interpreter", "sandbox_ipython"):
                 workdir = "/data"
 
+            # append system prompt
+            if id not in [
+                "openai_assistant",
+                "code_act",
+            ]:
+                if system_prompt:
+                    msg = ChatMessage(
+                        role=MessageRole.SYSTEM,
+                        content=system_prompt,
+                    )
+                    history.insert(0, msg)
+
+            # disable tools if cmd is not enabled
+            if not self.window.core.command.is_cmd():
+                tools = []
+                plugin_tools = []
+                plugin_specs = []
+
             provider = None
             agent = None
             if self.window.core.agents.provider.has(id):
@@ -123,6 +145,7 @@ class Runner:
                 "signals": signals,
                 "verbose": verbose,
             }
+
             if mode == AGENT_MODE_PLAN:
                 return self.run_plan(**kwargs)
             elif mode == AGENT_MODE_STEP:
@@ -164,27 +187,50 @@ class Runner:
         handler = agent.run(query, ctx=ctx, memory=memory, verbose=verbose)
         if verbose:
             print(f"User:  {query}")
+
         begin = True
+        item_ctx.live_output = ""  # to response append
+        item_ctx.output = ""  # empty to prevent render
+        item_ctx.stream = ""  # for stream
+
         async for event in handler.stream_events():
             if self.is_stopped():
-                self.send_live_clear(item_ctx, signals)
+                self.end_stream(item_ctx, signals)
                 break
             if isinstance(event, ToolCallResult):
-                output = f"\n-----------\nCode execution result:\n{event.tool_output}"
+                output = f"\n-----------\nExecution result:\n{event.tool_output}"
                 if verbose:
                     print(output)
-                self.send_live_append(item_ctx, signals, output, begin)
+                formatted = "\n```output\n" + str(event.tool_output) + "\n```\n"
+                item_ctx.live_output += formatted
+                item_ctx.stream = formatted
+                if item_ctx.stream_agent_output:
+                    self.send_stream(item_ctx, signals, begin)
             elif isinstance(event, ToolCall):
                 if "code" in event.tool_kwargs:
                     output = f"\n-----------\nTool call code:\n{event.tool_kwargs['code']}"
                     if verbose:
                         print(output)
-                    self.send_live_append(item_ctx, signals, output, begin)
+                    formatted = "\n```python\n" + str(event.tool_kwargs['code']) + "\n```\n"
+                    item_ctx.live_output += formatted
+                    item_ctx.stream = formatted
+                    if item_ctx.stream_agent_output:
+                        self.send_stream(item_ctx, signals, begin)
             elif isinstance(event, AgentStream):
                 if verbose:
                     print(f"{event.delta}", end="", flush=True)
-                self.send_live_append(item_ctx, signals, event.delta, begin)  # send stream to webview
-                begin = False
+                if event.delta:
+                    item_ctx.live_output += event.delta
+                    item_ctx.stream = event.delta
+                    if item_ctx.stream_agent_output:
+                        self.send_stream(item_ctx, signals, begin)  # send stream to webview
+                    begin = False
+            elif isinstance(event, AgentOutput):
+                thought, answer = self.extract_final_response(str(event))
+                if answer:
+                    item_ctx.set_agent_final_response(answer)
+                    if verbose:
+                        print(f"\nFinal response: {answer}")
 
         return await handler
 
@@ -216,7 +262,7 @@ class Runner:
         agent_ctx = Context(agent)
         memory = self.window.core.idx.chat.get_memory_buffer(history, llm)
         self.set_busy(signals)
-        response = await self.run_agent_workflow(
+        await self.run_agent_workflow(
             agent=agent,
             ctx=agent_ctx,
             query=prompt,
@@ -225,15 +271,28 @@ class Runner:
             item_ctx=ctx,
             signals=signals,
         )
-        response_ctx = self.add_ctx(ctx)
+        response_ctx = self.add_ctx(ctx, with_tool_outputs=True)
         response_ctx.set_input("inp")
-        response_ctx.set_output(str(response))
+
+        prev_output = ctx.live_output
+        # remove all <execute>...</execute>
+        if prev_output:
+            prev_output = re.sub(r'<execute>.*?</execute>', '', prev_output, flags=re.DOTALL)
+
+        response_ctx.set_agent_final_response(ctx.agent_final_response)  # always set to further use
+        response_ctx.set_output(prev_output)  # append from stream
         response_ctx.extra["agent_output"] = True  # mark as output response
         response_ctx.extra["agent_finish"] = True  # mark as finished
 
+        if ctx.agent_final_response:  # only if not empty
+            response_ctx.extra["output"] = ctx.agent_final_response
+
         # if there are tool outputs, img, files, append it to the response context
-        self.window.core.agents.tools.append_tool_outputs(response_ctx)
-        self.send_live_clear(response_ctx, signals)
+        if ctx.use_agent_final_response:
+            self.window.core.agents.tools.append_tool_outputs(response_ctx)
+        else:
+            self.window.core.agents.tools.extract_tool_outputs(response_ctx)
+        self.end_stream(response_ctx, signals)
 
         # send response
         self.send_response(response_ctx, signals, KernelEvent.APPEND_DATA)
@@ -338,6 +397,90 @@ class Runner:
         self.set_idle(signals)
         return True
 
+    def run_steps_once(
+            self,
+            agent: Any,
+            ctx: CtxItem,
+            prompt: str,
+            verbose: bool = False
+    ) -> Union[CtxItem, None]:
+        """
+        Run agent steps
+
+        :param agent: Agent instance
+        :param ctx: Input context
+        :param prompt: input text
+        :param verbose: verbose mode
+        :return: CtxItem with final response or True if stopped
+        """
+        if self.is_stopped():
+            return  # abort if stopped
+
+        is_last = False
+        task = agent.create_task(self.prepare_input(prompt))
+        tools_output = None
+
+        # run steps
+        i = 1
+        idx = 0
+        while not is_last:
+            if self.is_stopped():
+                break  # handle force stop
+
+            if verbose:
+                print ("\n----------- BEGIN STEP {} ----------\n".format(i))
+
+            step_output = agent.run_step(task.task_id)
+            is_last = step_output.is_last
+
+            # append each step to chat output, last step = final response, so we skip it
+            tools_output = self.window.core.agents.tools.export_sources(step_output.output)
+
+            if verbose:
+                print("\n")
+                print("Step: " + str(i))
+                print("Is last: " + str(is_last))
+                print("Tool calls: " + str(tools_output))
+                print("\n")
+
+            if not is_last:
+                step_ctx = self.add_ctx(ctx)
+                step_ctx.set_input(str(tools_output))
+                step_ctx.set_output("`{step_label} {i}`".format(
+                    step_label=trans('msg.agent.step'),
+                    i=str(i)
+                ))
+                step_ctx.cmds = tools_output
+                step_ctx.results = ctx.results  # get results from base ctx
+                ctx.results = []  # reset results
+
+                # copy extra data (output from plugins)
+                if tools_output:
+                    for k in ctx.extra:
+                        if not k.startswith("agent_"):
+                            step_ctx.extra[k] = ctx.extra[k]
+
+                # reset input ctx
+                for k in list(ctx.extra.keys()):
+                    if k != "agent_input":
+                        del ctx.extra[k]
+
+                step_ctx.extra["agent_step"] = True
+            i += 1
+            idx += 1
+
+        # final response
+        if is_last:
+            if self.is_stopped():
+                return  # abort if stopped
+            response = agent.finalize_response(task.task_id)
+            response_ctx = self.add_ctx(ctx)
+            response_ctx.set_input(str(tools_output))
+            response_ctx.set_output(str(response))
+            response_ctx.extra["agent_output"] = True  # mark as output response
+            response_ctx.extra["agent_finish"] = True  # mark as finished
+            return response_ctx
+
     def run_assistant(
             self,
             agent: Any,
@@ -482,7 +625,7 @@ class Runner:
 
             # this can be the last step in current sub-task, so send it first
             step_ctx = self.add_ctx(ctx)
-            step_ctx.set_input(str(tools_output)) # TODO: tool_outputs?
+            step_ctx.set_input(str(tools_output))  # TODO: tool_outputs?
             step_ctx.set_output("`{sub_task_label} {i}/{c}, {step_label} {j}`\n{task_header}".format(
                 sub_task_label=trans('msg.agent.plan.subtask'),
                 i=str(i),
@@ -534,7 +677,7 @@ class Runner:
             if self.is_stopped():
                 return True  # abort if stopped
 
-            # the is no tool calls here, only final response
+            # there is no tool calls here, only final response
             if step_output.is_last:
                 response = agent.finalize_response(task.task_id, step_output=step_output)
                 response_ctx = self.add_ctx(ctx)
@@ -546,6 +689,149 @@ class Runner:
 
         return True
 
+    def run_plan_once(
+            self,
+            agent: Any,
+            ctx: CtxItem,
+            prompt: str,
+            verbose: bool = False
+    ) -> Union[None, CtxItem]:
+        """
+        Run agent sub-tasks
+
+        :param agent: Agent instance
+        :param ctx: Input context
+        :param prompt: input text
+        :param verbose: verbose mode
+        :return: CtxItem with final response or None if stopped
+        """
+
+        verbose = True
+        if self.is_stopped():
+            return  # abort if stopped
+
+        plan_id = agent.create_plan(self.prepare_input(prompt))
+        plan = agent.state.plan_dict[plan_id]
+        c = len(plan.sub_tasks)
+
+        # prepare plan description
+        plan_desc = "`{current_plan}:`\n".format(
+            current_plan=trans('msg.agent.plan.current')
+        )
+        for sub_task in plan.sub_tasks:
+            plan_desc += "\n\n"
+            plan_desc += "\n**===== {sub_task_label}: {sub_task_name} =====**".format(
+                sub_task_label=trans('msg.agent.plan.subtask'),
+                sub_task_name=sub_task.name,
+            )
+            plan_desc += "\n{expected_label}: {expected_output}".format(
+                expected_label=trans('msg.agent.plan.expected'),
+                expected_output=str(sub_task.expected_output),
+            )
+            plan_desc += "\n{deps_label}: {dependencies}".format(
+                deps_label=trans('msg.agent.plan.deps'),
+                dependencies=str(sub_task.dependencies),
+            )
+
+        if verbose:
+            print(plan_desc)
+
+        plan_ctx = self.add_ctx(ctx)
+        plan_ctx.set_input("{num_subtasks_label}: {count}, {plan_label}: {plan_id}".format(
+            num_subtasks_label=trans('msg.agent.plan.num_subtasks'),
+            count=str(c),
+            plan_label=trans('msg.agent.plan'),
+            plan_id=plan_id,
+        ))
+        plan_ctx.set_output(plan_desc)
+        plan_ctx.cmds = []
+
+        all_steps_results = []
+
+        i = 1
+        for sub_task in plan.sub_tasks:
+            if self.is_stopped():
+                break
+
+            j = 1
+            task = agent.state.get_task(sub_task.name)
+            step_output = agent.run_step(task.task_id)
+            tools_output = self.window.core.agents.tools.export_sources(step_output.output)
+
+            task_header = "\n"
+            task_header += "\n**===== Sub Task {index}: {sub_task_name} =====**".format(
+                index=str(i),
+                sub_task_name=sub_task.name
+            )
+            task_header += "\nExpected output: {expected_output}".format(
+                expected_output=str(sub_task.expected_output)
+            )
+            task_header += "\nDependencies: {dependencies}".format(
+                dependencies=str(sub_task.dependencies)
+            )
+
+            if verbose:
+                print(task_header)
+
+            step_ctx = self.add_ctx(ctx)
+            step_ctx.set_input(str(tools_output))
+            step_ctx.set_output("`{sub_task_label} {i}/{c}, {step_label} {j}`\n{task_header}".format(
+                sub_task_label=trans('msg.agent.plan.subtask'),
+                i=str(i),
+                c=str(c),
+                step_label=trans('msg.agent.step'),
+                j=str(j),
+                task_header=task_header
+            ))
+            self.copy_step_results(step_ctx, ctx, tools_output)
+            step_ctx.extra["agent_step"] = True
+
+            all_steps_results.append(step_ctx)
+
+            # -----------------------------------------------------------
+            while not step_output.is_last:
+                if self.is_stopped():
+                    break
+
+                step_output = agent.run_step(task.task_id)
+                tools_output = self.window.core.agents.tools.export_sources(step_output.output)
+                j += 1
+                step_ctx = self.add_ctx(ctx)
+                step_ctx.set_input(str(tools_output))
+                step_ctx.set_output("`{sub_task_label} {i}/{c}, {step_label} {j}`\n{task_header}".format(
+                    sub_task_label=trans('msg.agent.plan.subtask'),
+                    i=str(i),
+                    c=str(c),
+                    step_label=trans('msg.agent.step'),
+                    j=str(j),
+                    task_header=task_header
+                ))
+                self.copy_step_results(step_ctx, ctx, tools_output)
+                step_ctx.extra["agent_step"] = True
+
+                all_steps_results.append(step_ctx)
+
+            extra = {}
+            extra["agent_output"] = True
+            if i == c:
+                extra["agent_finish"] = True
+
+            if self.is_stopped():
+                return  # abort if stopped
+
+            if step_output.is_last:
+                response = agent.finalize_response(task.task_id, step_output=step_output)
+                final_ctx = self.add_ctx(ctx)
+                final_ctx.set_input(str(tools_output))
+                final_ctx.set_output(str(response))
+                final_ctx.extra.update(extra)
+                all_steps_results.append(final_ctx)
+            i += 1
+
+        if all_steps_results:
+            return all_steps_results[-1]
+
+
     def copy_step_results(
             self,
             step_ctx: CtxItem,
@@ -634,6 +920,8 @@ class Runner:
         self.set_busy(signals)
         self.next_instruction = ""  # reset
         self.prev_score = -1  # reset
+
+        # run agent once
         self.run_once(prompt, tools, ctx.model)  # tool will update evaluation
         return self.handle_evaluation(ctx, self.next_instruction, self.prev_score, signals)
 
@@ -699,6 +987,7 @@ class Runner:
         context = BridgeContext()
         context.ctx = step_ctx
         context.history = self.window.core.ctx.all(meta_id=ctx.meta.id)
+        context.prompt = instruction  # use instruction as prompt
         preset = self.window.controller.presets.get_current()
         extra = {
             "agent_idx": preset.idx,
@@ -709,12 +998,14 @@ class Runner:
 
     def add_ctx(
             self,
-            from_ctx: CtxItem
+            from_ctx: CtxItem,
+            with_tool_outputs: bool = False
     ) -> CtxItem:
         """
         Add context item
 
         :param from_ctx: CtxItem (parent, source)
+        :param with_tool_outputs: True if tool outputs should be copied
         :return: CtxItem
         """
         ctx = CtxItem()
@@ -728,49 +1019,54 @@ class Runner:
         ctx.urls = from_ctx.urls  # copy from parent if appended from plugins
         ctx.attachments = from_ctx.attachments # copy from parent if appended from plugins
         ctx.live = True
+
+        if with_tool_outputs:
+            # copy tool outputs from parent context item
+            ctx.cmds = copy.deepcopy(from_ctx.cmds)
+            ctx.results = copy.deepcopy(from_ctx.results)
+            if "tool_output" in from_ctx.extra:
+                ctx.extra["tool_output"] = copy.deepcopy(from_ctx.extra["tool_output"])
         return ctx
 
-    def send_live_append(
+    def send_stream(
             self,
             ctx: CtxItem,
             signals: BridgeSignals,
-            text: str,
             begin: bool=False
     ):
         """
-        Send live response to chat window (BridgeSignals)
+        Send stream chunk to chat window (BridgeSignals)
 
         :param ctx: CtxItem
         :param signals: BridgeSignals
-        :param text: text to send
         :param begin: True if it is the beginning of the text
         """
-        context = BridgeContext()
-        context.ctx = ctx
-        extra = {
-            "chunk": str(text),
+        if signals is None:
+            return
+        chunk = ctx.stream.replace("<execute>", "\n```python\n").replace("</execute>", "\n```\n")
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+            "chunk": chunk, # use stream for live output
             "begin": begin,
         }
-        event = KernelEvent(KernelEvent.LIVE_APPEND, {
-            'context': context,
-            'extra': extra,
-        })
+        event = RenderEvent(RenderEvent.STREAM_APPEND, data)
         signals.response.emit(event)
 
-    def send_live_clear(self, ctx: CtxItem, signals: BridgeSignals):
+    def end_stream(self, ctx: CtxItem, signals: BridgeSignals):
         """
-        Send live clear response to chat window (BridgeSignals)
+        End of stream in chat window (BridgeSignals)
 
         :param ctx: CtxItem
         :param signals: BridgeSignals
         """
-        context = BridgeContext()
-        context.ctx = ctx
-        extra = {}
-        event = KernelEvent(KernelEvent.LIVE_CLEAR, {
-            'context': context,
-            'extra': extra,
-        })
+        if signals is None:
+            return
+        data = {
+            "meta": ctx.meta,
+            "ctx": ctx,
+        }
+        event = RenderEvent(RenderEvent.STREAM_END, data)
         signals.response.emit(event)
 
     def send_response(
@@ -788,6 +1084,8 @@ class Runner:
         :param event_name: kernel event
         :param kwargs: extra data
         """
+        if signals is None:
+            return
         context = BridgeContext()
         context.ctx = ctx
         event = KernelEvent(event_name, {
@@ -807,6 +1105,8 @@ class Runner:
         :param signals: BridgeSignals
         :param kwargs: extra data
         """
+        if signals is None:
+            return
         data = {
             "id": "agent",
             "msg": trans("status.agent.reasoning"),
@@ -826,6 +1126,8 @@ class Runner:
         :param signals: BridgeSignals
         :param kwargs: extra data
         """
+        if signals is None:
+            return
         data = {
             "id": "agent",
         }
@@ -844,6 +1146,8 @@ class Runner:
         :param signals: BridgeSignals
         :param msg: status message
         """
+        if signals is None:
+            return
         data = {
             "status": msg,
         }
@@ -877,3 +1181,14 @@ class Runner:
         :return: last exception or None if no error
         """
         return self.error
+
+    def extract_final_response(self, input_text: str) -> Tuple[str, str]:
+        pattern = r"\s*Thought:(.*?)Answer:(.*?)(?:$)"
+
+        match = re.search(pattern, input_text, re.DOTALL)
+        if not match:
+            return "", ""
+
+        thought = match.group(1).strip()
+        answer = match.group(2).strip()
+        return thought, answer
