@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.14 00:00:00                  #
+# Updated Date: 2025.07.25 22:00:00                  #
 # ================================================== #
 
 import copy
@@ -261,13 +261,15 @@ class Command:
     def unpack_tool_calls_chunks(
             self,
             ctx: CtxItem,
-            tool_calls: List[Dict[str, Any]]
+            tool_calls: List[Dict[str, Any]],
+            append_output: bool = False,
     ):
         """
         Handle / unpack tool calls
 
         :param ctx: context
         :param tool_calls: tool calls
+        :param append_output: if True then append output to context output
         """
         tmp_calls = []
         for tool_call in tool_calls:
@@ -276,6 +278,8 @@ class Command:
                     continue
                 if "arguments" not in tool_call["function"]:
                     continue
+                if not isinstance(tool_call["function"]["arguments"], str):
+                    continue
                 tool_call["function"]["arguments"] = json.loads(
                     tool_call["function"]["arguments"]
                 )
@@ -285,6 +289,12 @@ class Command:
                 print("Error parsing tool call JSON arguments: ", tool_call["function"]["arguments"])
         ctx.tool_calls = tmp_calls
 
+        if append_output:
+            # append tool calls to context output
+            ctx.extra["tool_calls"] = ctx.tool_calls
+            ctx.extra["tool_output"] = []
+
+
     def unpack_tool_calls_from_llama(
             self,
             tool_calls: List
@@ -410,17 +420,19 @@ class Command:
 
     def get_functions(
             self,
-            parent_id: Optional[str] = None
+            parent_id: Optional[str] = None,
+            force: bool = False
     ) -> List[Dict[str, Any]]:
         """
         Get current functions list
 
         :param parent_id: parent ID
+        :param force: force to get native functions
         :return: functions list
         """
         func = []
         func_user = self.window.controller.presets.get_current_functions()
-        if self.is_native_enabled():
+        if self.is_native_enabled(force=force):
             func = self.as_native_functions(all=False, parent_id=parent_id)
         if func_user is None:
             func_user = []
@@ -480,8 +492,10 @@ class Command:
         func_plugins = self.cmds_to_functions(cmds)  # plugin functions
         if self.window.controller.agent.legacy.enabled():
             func_agent = self.cmds_to_functions(self.window.controller.agent.legacy.get_functions())  # agent functions
-        if self.window.controller.agent.experts.enabled():
-            func_experts = self.cmds_to_functions(self.window.core.experts.get_functions())  # agent functions
+        if (self.window.controller.agent.experts.enabled()
+                or self.window.controller.agent.legacy.enabled(check_inline=False)):
+            if parent_id is None:  # don't append expert call tool for experts
+                func_experts = self.cmds_to_functions(self.window.core.experts.get_functions())  # agent functions
         return func_plugins + func_agent + func_experts
 
     def cmds_to_functions(
@@ -610,10 +624,11 @@ class Command:
                     pass
         return params
 
-    def is_native_enabled(self) -> bool:
+    def is_native_enabled(self, force: bool = False) -> bool:
         """
         Check if native tool calls are enabled
 
+        :param force: force check, ignore config
         :return: True if enabled
         """
         disabled_modes = [
@@ -622,33 +637,44 @@ class Command:
             MODE_COMPLETION,
         ]
         mode = self.window.core.config.get('mode')
+
+        # force disabled for specific modes
         if mode in disabled_modes:
-            return False  # disabled for specific modes
-        if self.window.controller.agent.legacy.enabled() or self.window.controller.agent.experts.enabled():
             return False
-        model = self.window.core.config.get('model')
-        if model:
-            model_data = self.window.core.models.get(model)
-            if model_data:
-                if not self.window.core.models.is_tool_call_allowed(mode, model_data):
-                    return False
-        enabled = self.window.core.config.get('func_call.native', False)  # otherwise check config
-        # if enabled:
-            # self.window.core.debug.info("[cmd] Native tool calls enabled")
-        return enabled
 
-    def is_cmd_prompt_enabled(self):
-        """
-        Check if command prompt is enabled
+        if not force:
+            # check model
+            model = self.window.core.config.get('model')
+            if model:
+                model_data = self.window.core.models.get(model)
+                if model_data:
+                    if not self.window.core.models.is_tool_call_allowed(mode, model_data):
+                        return False
 
-        :return: True if command prompt is enabled
+            # agents and experts
+            if self.window.controller.agent.legacy.enabled():
+                return self.window.core.config.get('agent.func_call.native', False)
+            if self.window.controller.agent.experts.enabled():
+                return self.window.core.config.get('experts.func_call.native', False)
+
+        # otherwise check config
+        return self.window.core.config.get('func_call.native', False)
+
+    def is_cmd(self):
         """
-        mode = self.window.core.config.get('mode')
-        return True
-        if mode == MODE_LLAMA_INDEX:
-            if self.window.controller.idx.index_selected():
-                return False
-        return True
+        Check if tool execute is enabled
+
+        :return: True if command is enabled
+        """
+        # check if cmd is enabled in config
+        if self.window.core.config.get('cmd'):
+            return True
+
+        # check if cmd inline plugin is enabled
+        if self.window.controller.plugins.is_type_enabled("cmd.inline"):
+            return True
+
+        return False
 
     def is_enabled(self, cmd: str) -> bool:
         """
