@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.07.13 01:00:00                  #
 # ================================================== #
 
 from typing import Optional, List
@@ -414,3 +414,42 @@ class BaseRenderer:
     def tool_output_end(self):
         """End tool output"""
         pass
+
+    def state_changed(
+            self,
+            state: str,
+            meta: CtxMeta,
+    ):
+        """
+        Render end
+
+        :param state: state name
+        :param meta: context meta
+        """
+        pass
+
+    def append_live(
+            self,
+            meta: CtxMeta,
+            ctx: CtxItem,
+            text_chunk: str,
+            begin: bool = False
+    ):
+        """
+        Append live output chunk to output
+
+        :param meta: context meta
+        :param ctx: context item
+        :param text_chunk: text chunk
+        :param begin: if it is the beginning of the text
+        """
+        pass
+
+    def clear_live(self, meta: CtxMeta, ctx: CtxItem):
+        """
+        Clear live output
+
+        :param meta: context meta
+        :param ctx: context item
+        """
+        pass
