@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 01:00:00                  #
+# Updated Date: 2025.07.22 22:00:00                  #
 # ================================================== #
 
 import os
@@ -163,7 +163,7 @@ class Body:
                 '<a href="extra-audio-read:{}" class="action-icon" data-id="{}" role="button"><span class="cmd">{}</span></a>'.format(
                     ctx.id,
                     ctx.id,
-                    self.get_icon("audio", trans("ctx.extra.audio"), ctx)
+                    self.get_icon("volume", trans("ctx.extra.audio"), ctx)
                 )
             )
             # copy ctx
@@ -205,7 +205,7 @@ class Body:
                     '<a href="extra-join:{}" class="action-icon edit-icon" data-id="{}" role="button"><span class="cmd">{}</span></a>'.format(
                         ctx.id,
                         ctx.id,
-                        self.get_icon("join", trans("ctx.extra.join"), ctx)
+                        self.get_icon("playlist_add", trans("ctx.extra.join"), ctx)
                     )
                 )
         return icons
@@ -224,7 +224,7 @@ class Body:
         :param item: context item
         :return: icon HTML
         """
-        icon = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "chat", icon + ".png")
+        icon = os.path.join(self.window.core.config.get_app_path(), "data", "icons", icon + ".svg")
         return '<img src="file://{}" class="action-img" title="{}" alt="{}" data-id="{}">'.format(
             icon, title, title, item.id)
 
@@ -381,7 +381,8 @@ class Body:
                 })
                 event.ctx = ctx
                 self.window.dispatch(event, all=True)  # handle by plugins
-                html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
+                if event.data['html']:
+                    html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
 
             # multiple tools, list
             elif "tool_output" in ctx.extra and isinstance(ctx.extra["tool_output"], list):
@@ -396,7 +397,8 @@ class Body:
                     })
                     event.ctx = ctx
                     self.window.dispatch(event, all=True)
-                    html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
+                    if event.data['html']:
+                        html += "<div class=\"tool-output-block\">" + event.data['html'] + "</div>"
             html += "</div>"
         return html
 
@@ -663,6 +665,7 @@ class Body:
             }
         }
         function showToolOutputLoader() {
+            return; // disabled
             const elements = document.querySelectorAll('.msg-bot');
             if (elements.length > 0) {
                 const last = elements[elements.length - 1];
@@ -860,6 +863,11 @@ class Body:
                 bridge.preview_text(text);
             }
         }
+        function bridgeRunCode(text) {
+            if (bridge) {
+                bridge.run_text(text);
+            }
+        }
         function bridgeUpdateScrollPosition(pos) {
             if (bridge) {
                 bridge.update_scroll_position(pos);
@@ -936,7 +944,18 @@ class Body:
                             }, 1000);
                         }
                     }                        
-                }                
+                }            
+                // btn run
+                const runButton = event.target.closest('.code-header-run');
+                if (runButton) {
+                    event.preventDefault();
+                    const parent = event.target.closest('.code-wrapper');
+                    const source = parent.querySelector('code');
+                    if (source) {
+                        const text = source.textContent || source.innerText;
+                        bridgeRunCode(text);
+                    }                        
+                }    
                 // btn preview
                 const previewButton = event.target.closest('.code-header-preview');
                 if (previewButton) {
