@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.12 00:00:00                  #
+# Updated Date: 2025.07.19 17:00:00                  #
 # ================================================== #
 
 import os
@@ -188,9 +188,10 @@ class Parser:
 
         :param soup: BeautifulSoup instance
         """
-        copy_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "chat", "copy.png")
-        preview_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "chat", "preview.png")
-        collapse_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "chat", "collapse.png")
+        copy_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "copy.svg")
+        preview_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "view.svg")
+        run_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "play.svg")
+        collapse_icon_path = os.path.join(self.window.core.config.get_app_path(), "data", "icons", "menu.svg")
 
         # syntax highlighting style
         style = self.window.core.config.get("render.code_syntax")
@@ -228,8 +229,15 @@ class Parser:
 
             # Get the class of <code> to determine the language (if available)
             code = el.find('code')
+            is_output = False
             language = code['class'][0] if (code and code.has_attr('class') and 'language-' in code['class'][0]) else ''
 
+            # tool output
+            if language == "language-output":
+                language = "language-python"
+                code['class'] = "language-python"
+                is_output = True
+
             lang_span = soup.new_tag('span', **{'class': "code-header-lang"})
             lang_span['class'] = "code-header-lang"
             if language:  # Strip 'language-' to get the actual language
@@ -238,6 +246,10 @@ class Parser:
             else:
                 lang_span.string = "code "
 
+            # if tool output, change the title to "output"
+            if is_output:
+                lang_span.string = "output"
+
             link_wrapper.append(lang_span)
 
             # html preview
@@ -249,6 +261,14 @@ class Parser:
                 preview.insert(0, preview_icon)
                 link_wrapper.append(preview)
 
+            elif language == 'python' and not is_output:
+                run = soup.new_tag('a', href=f'empty:{self.block_idx}')  # extra action link
+                run['class'] = "code-header-action code-header-run"
+                run.string = trans('ctx.extra.run')
+                run_icon = soup.new_tag('img', src=run_icon_path, **{'class': "action-img"})
+                run.insert(0, run_icon)
+                link_wrapper.append(run)
+
             link_wrapper.append(collapse)
             link_wrapper.append(copy)
             header.append(link_wrapper)
