@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.14 00:00:00                  #
+# Updated Date: 2025.07.25 06:00:00                  #
 # ================================================== #
 
 import json
@@ -252,6 +252,8 @@ class Renderer(BaseRenderer):
         :param ctx: context item
         """
         pid = self.get_or_create_pid(meta)
+        if pid is None:
+            return
         if self.window.controller.agent.legacy.enabled():
             if self.pids[pid].item is not None:
                 self.append_context_item(meta, self.pids[pid].item)
@@ -333,6 +335,17 @@ class Renderer(BaseRenderer):
 
         text = ctx.input
 
+        # if sub-reply
+        if isinstance(ctx.extra, dict) and "sub_reply" in ctx.extra and ctx.extra["sub_reply"]:
+            try:
+                json_encoded = json.loads(text)
+                if isinstance(json_encoded, dict):
+                    if "expert_id" in json_encoded and "result" in json_encoded:
+                        tmp = "@" + str(ctx.input_name) + ":\n\n" + str(json_encoded["result"])
+                        text = tmp
+            except json.JSONDecodeError:
+                pass
+
         # hidden internal call
         if ctx.internal \
                 and not ctx.first \
@@ -370,12 +383,21 @@ class Renderer(BaseRenderer):
         :param next_ctx: next context
         """
         self.tool_output_end()  # reset tools
-        if ctx.output is None or ctx.output == "":
-            return
+        output = ctx.output
+        if (isinstance(ctx.extra, dict)
+                and "output" in ctx.extra
+                and ctx.extra["output"]):
+            if self.window.core.config.get("llama.idx.chat.agent.render.all", False):
+                output = "__agent_begin__" + ctx.output + "__agent_end__" + ctx.extra["output"]
+            else:
+                output = ctx.extra["output"]
+        else:
+            if ctx.output is None or ctx.output == "":
+                return
         self.append_node(
             meta=meta,
             ctx=ctx,
-            html=ctx.output.strip(),
+            html=output.strip(),
             type=self.NODE_OUTPUT,
             prev_ctx=prev_ctx,
             next_ctx=next_ctx
@@ -956,15 +978,17 @@ class Renderer(BaseRenderer):
         if self.is_debug():
             debug = self.append_debug(ctx, pid, "input")
 
+        extra_style = ""
         extra = ""
         if ctx.extra is not None and "footer" in ctx.extra:
             extra = ctx.extra["footer"]
+            extra_style = "display:block;"
         html = (
             '<div class="msg-box msg-user" id="{msg_id}">'
             '<div class="name-header name-user">{name}</div>'
             '<div class="msg">'
             '{html}'
-            '<div class="msg-extra">{extra}</div>'
+            '<div class="msg-extra" style="{extra_style}">{extra}</div>'
             '{debug}'
             '</div>'
             '</div>'
@@ -973,6 +997,7 @@ class Renderer(BaseRenderer):
             name=name,
             html=html,
             extra=extra,
+            extra_style=extra_style,
             debug=debug,
         )
 
@@ -1007,9 +1032,9 @@ class Renderer(BaseRenderer):
         msg_id = "msg-bot-" + str(ctx.id) if ctx is not None else ""
         # if is_cmd:
         # html = self.helpers.format_cmd_text(html)
-        html = self.helpers.pre_format_text(html)
-        html = self.append_timestamp(ctx, html, type=self.NODE_OUTPUT)
+        html = self.helpers.pre_format_text(html)        
         html = self.parser.parse(html)
+        html = self.append_timestamp(ctx, html, type=self.NODE_OUTPUT)
         html = self.helpers.post_format_text(html)
         extra = self.append_extra(meta, ctx, footer=True, render=False)
         footer = self.body.prepare_action_icons(ctx)
@@ -1027,9 +1052,9 @@ class Renderer(BaseRenderer):
             .format(icon)
         )
         expand_btn = (
-            "<span class='toggle-cmd-output' onclick='toggleToolOutput({});' "
-            "role='button'>{} {}</span>"
-            .format(str(ctx.id), cmd_icon, trans('action.cmd.expand'))
+            "<span class='toggle-cmd-output' onclick='toggleToolOutput({});' title='{}' "
+            "role='button'>{}</span>"
+            .format(str(ctx.id), trans('action.cmd.expand'), cmd_icon)
         )
 
         # check if next ctx is internal and current ctx has commands
@@ -1075,7 +1100,6 @@ class Renderer(BaseRenderer):
                 )
 
         html_tools = (
-                spinner +
                 '<div class="tool-output" style="{}">'.format(output_class) +
                 expand_btn +
                 '<div class="content" style="display:none">' +
@@ -1335,6 +1359,8 @@ class Renderer(BaseRenderer):
         :param ctx: context item
         """
         pid = self.get_or_create_pid(meta)
+        if pid is None:
+            return
         if ctx.input_name is not None and ctx.input_name != "":
             self.pids[pid].name_user = ctx.input_name
         if ctx.output_name is not None and ctx.output_name != "":
