@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.01 01:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 OPENAI_DISABLE_TOOLS = [
@@ -23,4 +23,15 @@ OPENAI_REMOTE_TOOL_DISABLE_CODE_INTERPRETER = [
 ]
 OPENAI_REMOTE_TOOL_DISABLE_WEB_SEARCH = [
     "codex-mini-latest",
+]
+OPENAI_REMOTE_TOOL_DISABLE_COMPUTER_USE = [
+    "o4-mini-deep-research",
+    "o3-deep-research",
+    "codex-mini-latest",
+]
+OPENAI_REMOTE_TOOL_DISABLE_FILE_SEARCH = [
+    "codex-mini-latest",
+]
+OPENAI_REMOTE_TOOL_DISABLE_MCP = [
+    "codex-mini-latest",
 ]
\ No newline at end of file
