@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.14 00:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 import copy
@@ -66,6 +66,9 @@ class CtxItem:
         self.first = False
         self.live = False  # True if is current flow (not loaded from DB)
         self.agent_call = False # prevents plugin reply if True
+        self.agent_final_response = ""
+        self.use_agent_final_response = False  # use agent final response
+        self.stream_agent_output = True  # stream agent output in real-time
         self.tool_calls = []  # API tool calls
         self.index_meta = {}  # llama-index metadata ctx used
         self.doc_ids = []  # document ids
@@ -74,6 +77,7 @@ class CtxItem:
         self.sub_calls = 0  # sub calls count
         self.sub_call = False  # is sub call
         self.sub_reply = False  # sub call reply
+        self.sub_tool_call = False  # sub tool call
         self.hidden = False  # hidden context
         self.use_responses_api = False  # use responses API format
         self.pid = 0
@@ -83,6 +87,8 @@ class CtxItem:
         self.response = None  # response object
         self.async_disabled = False  # async disabled
         self.bag = None
+        self.live_output = ""
+        self.force_call = False  # force call even if command is not present
 
 
     @property
@@ -185,6 +191,14 @@ class CtxItem:
         self.output_name = name
         self.output_timestamp = int(time.time())
 
+    def set_agent_final_response(self, output: str):
+        """
+        Set agent final response
+
+        :param output: final response output
+        """
+        self.agent_final_response = output
+
     def set_tokens(self, input_tokens: int, output_tokens: int):
         """
         Set tokens usage
@@ -199,13 +213,14 @@ class CtxItem:
     def get_pid(self):
         return 0
 
-    def to_dict(self) -> dict:
+    def to_dict(self, dump: bool = False) -> dict:
         """
         Dump context item to dict
 
+        :param dump: if True, dump meta to dict
         :return: dict
         """
-        return {
+        data = {
             "id": self.id,
             "meta_id": self.meta_id,
             "idx": self.idx,
@@ -227,6 +242,7 @@ class CtxItem:
             "sub_calls": 0,
             "sub_call": False,
             "sub_reply": False,
+            "sub_tool_call": False,
             "extra": self.extra,
             "extra_ctx": self.extra_ctx,
             "cmds": self.cmds,
@@ -259,8 +275,16 @@ class CtxItem:
             "input_tokens": self.input_tokens,
             "output_tokens": self.output_tokens,
             "total_tokens": self.total_tokens,
+            "force_call": self.force_call,
             "meta": self.meta.to_dict() if type(self.meta) == CtxMeta else self.meta,
         }
+        if dump:
+            data["hidden"] = self.hidden
+            data["sub_calls"] = self.sub_calls
+            data["sub_call"] = self.sub_call
+            data["sub_reply"] = self.sub_reply
+            data["sub_tool_call"] = self.sub_tool_call
+        return data
 
     def from_dict(self, data: dict):
         """
@@ -316,24 +340,26 @@ class CtxItem:
         self.sub_calls = data.get("sub_calls", 0)
         self.sub_call = data.get("sub_call", False)
         self.sub_reply = data.get("sub_reply", False)
+        self.sub_tool_call = data.get("sub_tool_call", False)
         self.hidden = data.get("hidden", False)
 
 
-    def dump(self) -> str:
+    def dump(self, dump: bool = True) -> str:
         """
         Dump context item to JSON string
 
+        :param dump: if True, dump meta to dict
         :return: JSON string
         """
         try:
-            return json.dumps(self.to_dict())
+            return json.dumps(self.to_dict(dump), ensure_ascii=False, indent=2)
         except Exception as e:
             pass
         return ""
 
     def __str__(self):
         """To string"""
-        return self.dump()
+        return self.dump(True)
 
 
 class CtxMeta:
@@ -511,6 +537,22 @@ class CtxMeta:
     def get_pid(self):
         return 0
 
+    def dump(self) -> str:
+        """
+        Dump context item to JSON string
+
+        :return: JSON string
+        """
+        try:
+            return json.dumps(self.to_dict(), ensure_ascii=False, indent=2)
+        except Exception as e:
+            pass
+        return ""
+
+    def __str__(self):
+        """To string"""
+        return self.dump()
+
 class CtxGroup:
     def __init__(self, id=None, name=None):
         """
@@ -592,3 +634,19 @@ class CtxGroup:
         self.created = data.get("created", None)
         self.updated = data.get("updated", None)
         self.count = data.get("count", 0)
+
+    def dump(self) -> str:
+        """
+        Dump context item to JSON string
+
+        :return: JSON string
+        """
+        try:
+            return json.dumps(self.to_dict(), ensure_ascii=False, indent=2)
+        except Exception as e:
+            pass
+        return ""
+
+    def __str__(self):
+        """To string"""
+        return self.dump()
