@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.14 00:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
 
 import json
@@ -250,7 +250,9 @@ class ModelItem:
                 or self.id.startswith("o3")
                 or self.id.startswith("o4")
                 or self.id.startswith("o5")
-                or self.id.startswith("codex-")):
+                or self.id.startswith("codex-")
+                or self.id.startswith("dall-e-")
+                or self.id.startswith("computer-use")):
             return True
         return False
 
