@@ -6,10 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.11 19:00:00                  #
+# Updated Date: 2025.07.17 19:00:00                  #
 # ================================================== #
 
 import os
+import time
+import uuid
 
 from PySide6.QtCore import Slot
 
@@ -68,6 +70,19 @@ class Plugin(BasePlugin):
         """Initialize options"""
         self.config.from_defaults(self)
 
+    def make_temp_file_path(self, extension: str = "png"):
+        """
+        Make temporary file path for code execution
+
+        :param extension: file extension
+        :return: temporary file path
+        """
+        name = uuid.uuid4().hex + f".{extension}"
+        dir = os.path.join(self.window.core.config.get_user_dir('data'), "tmp")
+        if not os.path.exists(dir):
+            os.makedirs(dir, exist_ok=True)
+        return os.path.join(dir, name)
+
     def handle(self, event: Event, *args, **kwargs):
         """
         Handle dispatched event
@@ -213,6 +228,8 @@ class Plugin(BasePlugin):
 
             # connect signals
             worker.signals.output.connect(self.handle_interpreter_output)
+            worker.signals.output_begin.connect(self.handle_interpreter_output_begin)
+            worker.signals.output_end.connect(self.handle_interpreter_output_end)
             worker.signals.clear.connect(self.handle_interpreter_clear)
             worker.signals.html_output.connect(self.handle_html_output)
             worker.signals.ipython_output.connect(self.handle_ipython_output)
@@ -241,7 +258,7 @@ class Plugin(BasePlugin):
         # print(data)
         cleaned_data = self.get_interpreter().remove_ansi(data)
         self.window.tools.get("interpreter").append_output(cleaned_data)
-        if self.window.tools.get("interpreter").opened:
+        if self.window.tools.get("interpreter").is_opened():
             self.window.update_status("")
 
     @Slot(object, str)
@@ -256,6 +273,24 @@ class Plugin(BasePlugin):
             return
         self.window.tools.get("interpreter").append_output(data, type)
 
+    @Slot(str)
+    def handle_interpreter_output_begin(self, type: str):
+        """
+        Handle interpreter output begin
+
+        :param type: output type
+        """
+        self.window.tools.get("interpreter").output_begin(type)
+
+    @Slot(str)
+    def handle_interpreter_output_end(self, type: str):
+        """
+        Handle interpreter output end
+
+        :param type: output type
+        """
+        self.window.tools.get("interpreter").output_end(type)
+
     @Slot()
     def handle_interpreter_clear(self):
         """Handle interpreter clear"""
@@ -271,6 +306,39 @@ class Plugin(BasePlugin):
         self.window.tools.get("html_canvas").set_output(data)
         self.window.tools.get("html_canvas").auto_open()
 
+    @Slot(str)
+    def handle_python_run(self, code: str):
+        """
+        Handle Python code run
+
+        :param code: Python code to execute
+        """
+        cmd = "code_execute"
+        if self.window.tools.get("interpreter").is_ipython():
+            cmd = "ipython_execute"
+            if self.get_option_value("fresh_kernel"):
+                self.get_interpreter().restart_kernel()
+                time.sleep(1)
+        self.window.tools.get("interpreter").clear_output()
+        commands = [
+            {
+                "cmd": cmd,
+                "params": {
+                    "code": code,
+                    "path": ".interpreter.current.py",
+                },
+                "silent": True,
+                "force": True,
+            }
+        ]
+        event = Event(Event.CMD_EXECUTE, {
+            'commands': commands,
+            'silent': True,
+        })
+        event.ctx = CtxItem()  # tmp
+        self.handle(event)
+        self.window.tools.get("interpreter").auto_open()
+
     def get_interpreter(self):
         """
         Get interpreter
