@@ -6,13 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.01 01:00:00                  #
+# Updated Date: 2025.07.26 18:00:00                  #
 # ================================================== #
-import base64
 
-from httpx_socks import SyncProxyTransport
-
-from openai import OpenAI, DefaultHttpxClient
+from openai import OpenAI
 
 from pygpt_net.core.types import (
     MODE_ASSISTANT,
@@ -22,22 +19,27 @@ from pygpt_net.core.types import (
     MODE_IMAGE,
     MODE_VISION,
     MODE_RESEARCH,
+    MODE_COMPUTER,
 )
 from pygpt_net.core.bridge.context import BridgeContext
+from pygpt_net.item.model import ModelItem
 
 from .audio import Audio
 from .assistants import Assistants
 from .chat import Chat
 from .completion import Completion
+from .computer import Computer
+from .container import Container
 from .image import Image
 from .responses import Responses
 from .store import Store
 from .summarizer import Summarizer
+from .tools import Tools
 from .vision import Vision
-from pygpt_net.item.model import ModelItem
 
 
 class Gpt:
+
     def __init__(self, window=None):
         """
         OpenAI API wrapper core
@@ -49,10 +51,13 @@ class Gpt:
         self.audio = Audio(window)
         self.chat = Chat(window)
         self.completion = Completion(window)
+        self.container = Container(window)
+        self.computer = Computer(window)
         self.image = Image(window)
         self.responses = Responses(window)
         self.store = Store(window)
         self.summarizer = Summarizer(window)
+        self.tools = Tools(window)
         self.vision = Vision(window)
 
     def get_client(
@@ -67,26 +72,8 @@ class Gpt:
         :param model: Model
         :return: OpenAI client
         """
-        args = {
-            "api_key": self.window.core.config.get('api_key'),
-            "organization": self.window.core.config.get('organization_key'),
-        }
-        # api endpoint
-        if self.window.core.config.has('api_endpoint'):
-            endpoint = self.window.core.config.get('api_endpoint')
-            if endpoint:
-                args["base_url"] = endpoint
-        # proxy
-        if self.window.core.config.has('api_proxy'):
-            proxy = self.window.core.config.get('api_proxy')
-            if proxy:
-                transport = SyncProxyTransport.from_url(proxy)
-                args["http_client"] = DefaultHttpxClient(
-                    transport=transport,
-                )
-
         # update client args by mode and model
-        args = self.window.core.models.prepare_client_args(args, mode, model)
+        args = self.window.core.models.prepare_client_args(mode, model)
         return OpenAI(**args)
 
     def call(self, context: BridgeContext, extra: dict = None) -> bool:
@@ -98,6 +85,7 @@ class Gpt:
         :return: result
         """
         mode = context.mode
+        parent_mode = context.parent_mode  # real mode (global)
         prompt = context.prompt
         stream = context.stream
         model = context.model  # model instance (item, not id)
@@ -105,13 +93,15 @@ class Gpt:
         assistant_id = context.assistant_id
         tools_outputs = context.tools_outputs
         max_tokens = context.max_tokens  # max output tokens
+        is_expert_call = context.is_expert_call
 
         ctx = context.ctx
         ai_name = ctx.output_name
         thread_id = ctx.thread  # from ctx
 
-        # --- Responses API ---- /beta/
-        use_responses_api = False
+        # --- Responses API ----
+        use_responses_api = self.responses.is_enabled(model, mode, parent_mode, is_expert_call)
+        ctx.use_responses_api = use_responses_api  # set in context
 
         # get model id
         model_id = None
@@ -120,13 +110,6 @@ class Gpt:
             if max_tokens > model.tokens:  # check max output tokens
                 max_tokens = model.tokens
 
-            if model.is_gpt():
-                if (mode in [MODE_CHAT, MODE_RESEARCH]
-                        and self.window.core.config.get('api_use_responses', False)):
-                    use_responses_api = True  # use responses API for chat mode, only OpenAI models
-
-        ctx.use_responses_api = use_responses_api  # set in context
-
         response = None
         used_tokens = 0
         context.max_tokens = max_tokens  # update max output tokens
@@ -144,7 +127,8 @@ class Gpt:
         elif mode in [
             MODE_CHAT,
             MODE_AUDIO,
-            MODE_RESEARCH
+            MODE_RESEARCH,
+            MODE_COMPUTER,
         ]:
             # responses API
             if use_responses_api:
@@ -204,12 +188,15 @@ class Gpt:
                 )
             return True  # if assistant mode then return here, will be handled async
 
-        # if stream
+        # ------- streaming response -------
+
         if stream:
             ctx.stream = response  # generator
             ctx.set_output("", ai_name)  # set empty output
             ctx.input_tokens = used_tokens  # get from input tokens calculation
-            return True
+            return True  # <-- end here, will be handled in chat stream controller
+
+        # ------- non-streaming response -------
 
         if response is None:
             return False
@@ -219,76 +206,14 @@ class Gpt:
             print("Error in GPT response: " + str(response["error"]))
             return False
 
-        # get output text from response (not-stream mode)
-        output = ""
-        if mode == MODE_COMPLETION:
-            output = response.choices[0].text.strip()
-        elif mode in [
-            MODE_CHAT, 
-            MODE_VISION, 
-            MODE_RESEARCH
-        ]:
-            if use_responses_api:
-                if response.output_text:
-                    output = response.output_text.strip()
-                if response.output:
-                    ctx.tool_calls = self.window.core.command.unpack_tool_calls_responses(
-                        response.output,
-                    )
-            else:
-                if response.choices[0]:
-                    print(response.choices[0].message)
-                    if response.choices[0].message.content:
-                        output = response.choices[0].message.content.strip()
-                    elif response.choices[0].message.tool_calls:
-                        ctx.tool_calls = self.window.core.command.unpack_tool_calls(
-                            response.choices[0].message.tool_calls,
-                        )
-        # audio
-        elif mode in [MODE_AUDIO]:
-            if response.choices[0]:
-                if response.choices[0].message and response.choices[0].message.audio:
-                    ctx.audio_output = response.choices[0].message.audio.data
-                    ctx.audio_id = response.choices[0].message.audio.id
-                    ctx.audio_expires_ts = response.choices[0].message.audio.expires_at
-                    ctx.is_audio = True
-                    output = response.choices[0].message.audio.transcript  # from transcript
-                    self.chat.audio_prev_expires_ts = ctx.audio_expires_ts
-                    self.chat.audio_prev_id = ctx.audio_id
-                elif response.choices[0].message and not response.choices[0].message.audio:
-                    output = response.choices[0].message.content
-                    ctx.audio_id = self.chat.audio_prev_id
-                    ctx.audio_expires_ts = self.chat.audio_prev_expires_ts
-                if response.choices[0].message.tool_calls:
-                    ctx.tool_calls = self.window.core.command.unpack_tool_calls(
-                        response.choices[0].message.tool_calls,
-                    )
-
-        ctx.set_output(output, ai_name)
+        ctx.ai_name = ai_name
 
+        # post-unpack response data, like: output text, tool calls, images, file citations, etc.
         if not use_responses_api:
-            ctx.set_tokens(
-                response.usage.prompt_tokens,
-                response.usage.completion_tokens,
-            )
+            self.chat.unpack_response(mode, response, ctx)
         else:
-            ctx.set_tokens(
-                response.usage.input_tokens,
-                response.usage.output_tokens,
-            )
-            if mode == MODE_CHAT:
-                # if image generation call in responses API
-                image_data = [
-                    output.result
-                    for output in response.output
-                    if output.type == "image_generation_call"
-                ]
-                if image_data:
-                    img_path = self.window.core.image.gen_unique_path(ctx)
-                    image_base64 = image_data[0]
-                    with open(img_path, "wb") as f:
-                        f.write(base64.b64decode(image_base64))
-                    ctx.images = [img_path]
+            self.responses.unpack_response(mode, response, ctx)
+
         return True
 
     def quick_call(self, context: BridgeContext, extra: dict = None) -> str:
@@ -299,11 +224,21 @@ class Gpt:
         :param extra: Extra arguments
         :return: response content
         """
+        # if normal request call then redirect
+        if context.request:
+            context.stream = False
+            context.mode = "chat"  # fake mode for redirect
+            result = self.call(context, extra)
+            return context.ctx.output
+
+        ctx = context.ctx
         mode = context.mode
         prompt = context.prompt
         system_prompt = context.system_prompt
         max_tokens = context.max_tokens
         temperature = context.temperature
+        functions = context.external_functions
+        history = context.history
         model = context.model
         if model is None:
             model = self.window.core.models.from_defaults()
@@ -311,10 +246,26 @@ class Gpt:
         client = self.get_client(mode, model)
         messages = []
         messages.append({"role": "system", "content": system_prompt})
+
+        if history:
+            for item in history:
+                messages.append({
+                    "role": "user",
+                    "content": str(item.final_input)
+                })
+                messages.append({
+                    "role": "assistant",
+                    "content": str(item.final_output)
+                })
         messages.append({"role": "user", "content": prompt})
         additional_kwargs = {}
         if max_tokens > 0:
             additional_kwargs["max_tokens"] = max_tokens
+
+        # tools / functions
+        tools = self.window.core.gpt.tools.prepare(model, functions)
+        if len(tools) > 0:
+            additional_kwargs["tools"] = tools
         
         try:
             response = client.chat.completions.create(
@@ -326,6 +277,11 @@ class Gpt:
                 presence_penalty=0.0,
                 **additional_kwargs,
             )
+            # extract tool calls
+            if ctx and response.choices[0].message.tool_calls:
+                ctx.tool_calls = self.window.core.command.unpack_tool_calls(
+                    response.choices[0].message.tool_calls,
+                )
             return response.choices[0].message.content
         except Exception as e:
             self.window.core.debug.log(e)
