@@ -6,9 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 16:00:00                  #
+# Updated Date: 2025.07.20 16:00:00                  #
 # ================================================== #
 
+import json
 import os
 from time import strftime
 from typing import Dict
@@ -20,9 +21,10 @@ from PySide6.QtWidgets import QWidget
 from pygpt_net.core.tabs.tab import Tab
 from pygpt_net.tools.base import BaseTool
 from pygpt_net.tools.code_interpreter.ui.dialogs import Tool
-from pygpt_net.core.events import Event, KernelEvent
+from pygpt_net.core.events import Event, KernelEvent, BaseEvent, RenderEvent
 from pygpt_net.item.ctx import CtxItem
 from pygpt_net.utils import trans
+from .ui.html import HtmlOutput, CodeBlock
 
 from .ui.widgets import PythonInput, PythonOutput, ToolWidget, ToolSignals
 
@@ -44,12 +46,18 @@ class CodeInterpreter(BaseTool):
         self.auto_clear = False
         self.dialog = None
         self.ipython = True
+        self.auto_opened = False
         self.signals = ToolSignals()
 
         # interpreter data files in /data directory
         self.file_current = ".interpreter.current.py"
         self.file_input = ".interpreter.input.py"
         self.file_output = ".interpreter.output.py"
+        self.file_output_json = ".interpreter.output.json"
+
+        # buffers
+        self.input_buffer = ""
+        self.output_buffer = ""
 
     def setup(self):
         """Setup"""
@@ -74,6 +82,16 @@ class CodeInterpreter(BaseTool):
             self.set_initial_size()
             self.window.core.config.set("interpreter.dialog.initialized", True)
 
+    def handle(self, event: BaseEvent):
+        """
+        Handle event
+
+        :param event: Event instance
+        """
+        name = event.name
+        if name == RenderEvent.ON_THEME_CHANGE:
+            self.reload_view()  # reload web view on theme change
+
     def set_initial_size(self):
         """Set default sizes"""
         def set_initial_splitter_height():
@@ -86,6 +104,16 @@ class CodeInterpreter(BaseTool):
                 QTimer.singleShot(0, set_initial_splitter_height)
         QTimer.singleShot(0, set_initial_splitter_height)
 
+        def set_initial_splitter_dialog_height():
+            total_height = self.window.ui.splitters['interpreter_dialog'].size().height()
+            if total_height > 0:
+                size_output = int(total_height * 0.85)
+                size_input = total_height - size_output
+                self.window.ui.splitters['interpreter_dialog'].setSizes([size_output, size_input])
+            else:
+                QTimer.singleShot(0, set_initial_splitter_dialog_height)
+        QTimer.singleShot(0, set_initial_splitter_dialog_height)
+
         def set_initial_splitter_width():
             total_width = self.window.ui.splitters['interpreter.columns'].size().width()
             if total_width > 0:
@@ -107,6 +135,11 @@ class CodeInterpreter(BaseTool):
     def on_reload(self):
         """On app profile reload"""
         self.setup()
+        self.reload_view()
+
+    def reload_view(self):
+        """Reload view"""
+        self.signals.reload_view.emit()
 
     def update(self):
         """Update menu"""
@@ -121,15 +154,56 @@ class CodeInterpreter(BaseTool):
             self.window.ui.menu['tools.interpreter'].setChecked(False)
         """
 
-    def append_output(self, output: str, type="stdout", **kwargs):
+    def output_begin(self, type: str = "stdout"):
+        """
+        Begin output
+
+        :param type: Output type (stdout, stderr, etc.)
+        """
+        if type == "stdout":
+            self.output_buffer = ""
+        elif type == "stdin":
+            self.input_buffer = ""
+        self.signals.begin_output.emit(type)
+
+    def output_end(self, type: str = "stdout"):
+        """
+        End output
+
+        :param type: Output type (stdout, stderr, etc.)
+        """
+        # extract data files from output buffer
+        ctx = CtxItem()
+        images = []
+        files = []
+        if type == "stdout":
+            if self.output_buffer:
+                self.window.core.filesystem.parser.extract_data_files(ctx, self.output_buffer)
+                if ctx.images:
+                    images = ctx.images
+                if ctx.files:
+                    files = ctx.files
+            self.output_buffer = ""  # clear buffer
+        elif type == "stdin":
+            self.input_buffer = ""
+        self.signals.end_output.emit(type, ctx)
+
+        if images or files:
+            self.save_output_nodes()  # update nodes
+
+    def append_output(self, data: str, type="stdout", **kwargs):
         """
         Append output to the interpreter window
 
-        :param output: Output data
+        :param data: Data to append
         :param type: Output type
         :param kwargs: Additional parameters
         """
-        self.signals.update.emit(output, type, True)
+        if type in ["stdout", "stderr"]:
+            self.output_buffer += str(data)
+        elif type == "stdin":
+            self.input_buffer += str(data)
+        self.signals.update.emit(data, type, True)
         self.save_output()
         self.load_history()
 
@@ -149,6 +223,14 @@ class CodeInterpreter(BaseTool):
         """
         return os.path.join(self.window.core.config.get_user_dir("data"), self.file_output)
 
+    def get_path_output_json(self) -> str:
+        """
+        Get output path for JSON files
+
+        :return: Output path
+        """
+        return os.path.join(self.window.core.config.get_user_dir("data"), self.file_output_json)
+
     def get_widget(self) -> ToolWidget:
         """
         Get tool widget
@@ -157,7 +239,7 @@ class CodeInterpreter(BaseTool):
         """
         return self.dialog.widget
 
-    def get_widget_history(self) -> PythonOutput:
+    def get_widget_history(self) -> HtmlOutput:
         """
         Get history widget
 
@@ -165,7 +247,7 @@ class CodeInterpreter(BaseTool):
         """
         return self.dialog.widget.history
 
-    def get_widget_output(self) -> PythonOutput:
+    def get_widget_output(self) -> HtmlOutput:
         """
         Get output widget
 
@@ -205,16 +287,56 @@ class CodeInterpreter(BaseTool):
     def load_output(self):
         """Load output data from file"""
         data = self.get_output()
-        self.signals.update.emit(data, "stdout", False)
+        # self.signals.update.emit(data, "stdout", False)
         self.signals.focus_input.emit()
 
+        # from nodes
+        items = []
+        path_json = self.get_path_output_json()
+        if os.path.exists(path_json):
+            with open(path_json, "r", encoding="utf-8") as f:
+                try:
+                    items = json.load(f)
+                except:
+                    pass
+        nodes = []
+        for item in items:
+            node = CodeBlock(
+                content=item.get("content", ""),
+                type=item.get("type", "text"),
+                images=item.get("images", []),
+                files=item.get("files", []),
+            )
+            nodes.append(node)
+        self.signals.restore_nodes.emit(nodes)
+
     def save_output(self):
         """Save output data to file"""
         path = self.get_path_output()
-        data = self.get_widget_output().toPlainText()
+        data = self.get_widget_output().get_plaintext()
         with open(path, "w", encoding="utf-8") as f:
             f.write(data)
 
+        # save nodes
+        self.save_output_nodes()
+
+    def save_output_nodes(self):
+        """Save output nodes to file"""
+        nodes = self.get_widget_output().get_nodes()
+        items = []
+        for node in nodes:
+            item = {
+                "content": node.content,
+                "type": node.type,
+                "images": node.images,
+                "files": node.files,
+            }
+            items.append(item)
+        data = json.dumps(items, indent=4, ensure_ascii=False)
+        path_json = self.get_path_output_json()
+        with open(path_json, "w", encoding="utf-8") as f:
+            f.write(data)
+
     def get_history(self) -> str:
         """
         Load and get history from file
@@ -253,6 +375,9 @@ class CodeInterpreter(BaseTool):
         path = self.get_path_output()
         if os.path.exists(path):
             os.remove(path)
+        path_json = self.get_path_output_json()
+        if os.path.exists(path_json):
+            os.remove(path_json)
         self.signals.clear_output.emit()
 
     def clear(self, force: bool = False):
@@ -277,6 +402,7 @@ class CodeInterpreter(BaseTool):
 
     def restart_kernel(self):
         """Restart kernel"""
+        self.window.controller.kernel.resume()
         print("Sending restart event...")
         commands = [
             {
@@ -298,12 +424,29 @@ class CodeInterpreter(BaseTool):
         })
         self.window.dispatch(event)
 
+    def is_ipython(self) -> bool:
+        """
+        Check if ipython is enabled
+
+        :return: True if ipython is enabled, False otherwise
+        """
+        return self.ipython
+
+    def is_opened(self) -> bool:
+        """
+        Check if interpreter dialog is opened
+
+        :return: True if dialog is opened, False otherwise
+        """
+        return self.opened
+
     def send_input(self, widget: ToolWidget):
         """
         Send input to interpreter
 
         :param widget: PythonInput widget
         """
+        self.window.controller.kernel.resume()
         self.store_history(widget)
         if self.auto_clear:
             self.clear_output()
@@ -311,12 +454,15 @@ class CodeInterpreter(BaseTool):
         input_textarea = widget.input
         input = str(input_textarea.toPlainText()).strip()
 
-        if input == "/restart":
+        # replace tab with 4 spaces
+        input = input.replace("\t", "    ")
+
+        if input.strip().startswith("/restart"):
             self.restart_kernel()
             input_textarea.clear()
             input_textarea.setFocus()
             return
-        elif input == "/clear":
+        elif input.strip().startswith("/clear"):
             self.clear(force=True)
             input_textarea.clear()
             input_textarea.setFocus()
@@ -332,6 +478,51 @@ class CodeInterpreter(BaseTool):
         if self.ipython:
             cmd = "ipython_execute"
 
+        commands = [
+            {
+                "cmd": cmd,
+                "params": {
+                    "code": input,
+                    "path": self.file_current,
+                    "auto_init": True,  # auto initialize kernel if not initialized after error
+                },
+                "silent": True,
+                "force": True,
+            }
+        ]
+        event = Event(Event.CMD_EXECUTE, {
+            'commands': commands,
+            'silent': True,
+        })
+        event.ctx = CtxItem()  # tmp
+        self.window.controller.command.dispatch_only(event)
+        input_textarea.clear()
+        input_textarea.setFocus()
+
+    def run_input(self, input: str):
+        """
+        Run input code directly
+
+        :param input: Code to execute
+        """
+        self.window.controller.kernel.resume()
+        if input == "/restart":
+            self.restart_kernel()
+            return
+        elif input == "/clear":
+            self.clear(force=True)
+            return
+
+        if self.is_all():
+            cmd = "code_execute_all"
+        else:
+            if input == "":
+                return
+            cmd = "code_execute"
+
+        if self.ipython:
+            cmd = "ipython_execute"
+
         commands = [
             {
                 "cmd": cmd,
@@ -349,8 +540,6 @@ class CodeInterpreter(BaseTool):
         })
         event.ctx = CtxItem()  # tmp
         self.window.controller.command.dispatch_only(event)
-        input_textarea.clear()
-        input_textarea.setFocus()
 
     def update_input(self):
         """Update input data"""
@@ -386,16 +575,18 @@ class CodeInterpreter(BaseTool):
         """
         if not widget.history:
             return
-        data = widget.history.toPlainText()
+        data = widget.history.get_plaintext()
         self.save_history(data)
 
     def open(self):
         """Open interpreter dialog"""
         self.opened = True
+        self.auto_opened = False
         self.load_history()
         self.load_output()
         self.window.ui.dialogs.open('interpreter', width=800, height=600)
         self.get_widget_input().setFocus()
+        self.dialog.widget.on_open()
         self.cursor_to_end()
         self.scroll_to_bottom()
         self.update()
@@ -406,14 +597,31 @@ class CodeInterpreter(BaseTool):
         self.window.ui.dialogs.close('interpreter')
         self.update()
 
+    def auto_open(self):
+        """Auto open dialog or tab"""
+        if self.window.controller.ui.tabs.is_current_tool(self.id):
+            tool_col = self.window.controller.ui.tabs.get_tool_column(self.id)
+            current_col = self.window.controller.ui.tabs.column_idx
+            if tool_col == 1 and tool_col != current_col:
+                self.window.controller.ui.tabs.enable_split_screen(True)  # enable split screen
+            return # do not open if already opened in tab
+        elif self.window.controller.ui.tabs.is_tool(self.id):
+            tab = self.window.controller.ui.tabs.get_first_tab_by_tool(self.id)
+            if tab:
+                tool_col = tab.column_idx
+                current_col = self.window.controller.ui.tabs.column_idx
+                self.window.controller.ui.tabs.switch_tab_by_idx(tab.idx, tab.column_idx)
+                if tool_col == 1 and tool_col != current_col:
+                    self.window.controller.ui.tabs.enable_split_screen(True)  # enable split screen
+                return # do not open if already opened in tab
+        if not self.auto_opened:
+            self.auto_opened = True
+            self.open()
+
     def scroll_to_bottom(self):
         """Scroll down"""
-        self.get_widget_history().verticalScrollBar().setValue(
-            self.get_widget_history().verticalScrollBar().maximum()
-        )
-        self.get_widget_output().verticalScrollBar().setValue(
-            self.get_widget_output().verticalScrollBar().maximum()
-        )
+        self.get_widget_history().scroll_to_bottom()
+        self.get_widget_output().scroll_to_bottom()
 
     def toggle(self):
         """Toggle interpreter dialog open/close"""
@@ -509,9 +717,7 @@ class CodeInterpreter(BaseTool):
 
     def cursor_to_end(self):
         """Move cursor to end"""
-        cur = self.get_widget_history().textCursor()
-        cur.movePosition(QTextCursor.End)
-        self.get_widget_history().setTextCursor(cur)
+        self.get_widget_history().scroll_to_bottom()
 
     def setup_menu(self) -> Dict[str, QAction]:
         """
