@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.12 00:00:00                  #
+# Updated Date: 2025.07.16 15:00:00                  #
 # ================================================== #
 
 import re
@@ -16,6 +16,7 @@ from PySide6.QtWebChannel import QWebChannel
 from PySide6.QtWebEngineCore import QWebEngineSettings, QWebEnginePage
 from PySide6.QtWebEngineWidgets import QWebEngineView
 from PySide6.QtGui import QAction, QIcon, QKeySequence
+from PySide6.QtWidgets import QMenu
 
 from pygpt_net.core.events import RenderEvent
 from pygpt_net.item.ctx import CtxMeta
@@ -78,7 +79,7 @@ class HtmlOutput(QWebEngineView):
         self.html_content = html
 
         # set page HTML
-        self.setHtml(html)
+        self.setHtml(html, baseUrl="file://")
 
     def get_html_content(self) -> str:
         """
@@ -94,12 +95,7 @@ class HtmlOutput(QWebEngineView):
 
         :param position: position
         """
-        menu = self.createStandardContextMenu()
-
-        # remove defaults
-        for action in menu.actions()[::-1]:
-            menu.removeAction(action)
-
+        menu = QMenu(self)
         selected_text = ""
         is_selection = self.page().hasSelection()
         if is_selection:
@@ -111,6 +107,13 @@ class HtmlOutput(QWebEngineView):
             action.triggered.connect(self.copy_selected_text)
             menu.addAction(action)
 
+            # audio read
+            action = QAction(QIcon(":/icons/volume.svg"), trans('text.context_menu.audio.read'), self)
+            action.triggered.connect(
+                lambda: self.signals.audio_read.emit(selected_text)
+            )
+            menu.addAction(action)
+
             # copy to
             copy_to_menu = self.window.ui.context_menu.get_copy_to_menu(self, selected_text)
             menu.addMenu(copy_to_menu)
@@ -127,6 +130,13 @@ class HtmlOutput(QWebEngineView):
             action.triggered.connect(self.select_all_text)
             menu.addAction(action)
 
+            # save as (all) - plain
+            action = QAction(QIcon(":/icons/save.svg"), trans('action.save_as') + " (text)", self)
+            action.triggered.connect(
+                lambda: self.signals.save_as.emit(re.sub(r'\n{2,}', '\n\n', self.plain), 'txt')
+            )
+            menu.addAction(action)
+
             # save as (all) - html
             action = QAction(QIcon(":/icons/save.svg"), trans('action.save_as') + " (html)", self)
             action.triggered.connect(
@@ -136,7 +146,7 @@ class HtmlOutput(QWebEngineView):
 
         action = QAction(QIcon(":/icons/search.svg"), trans('text.context_menu.find'), self)
         action.triggered.connect(self.find_open)
-        action.setShortcut(QKeySequence("Ctrl+F"))
+        # action.setShortcut(QKeySequence("Ctrl+F"))
         menu.addAction(action)
 
         menu.exec_(self.mapToGlobal(position))
@@ -230,6 +240,9 @@ class HtmlOutput(QWebEngineView):
             if self.tab:
                 col_idx = self.tab.column_idx
                 self.window.controller.ui.tabs.on_column_focus(col_idx)
+        elif event.type() == QEvent.KeyPress:
+            if event.key() == Qt.Key_F and (event.modifiers() & Qt.ControlModifier):
+                self.find_open()
         return super().eventFilter(source, event)
 
 
