@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.09 00:00:00                  #
+# Updated Date: 2025.07.22 15:00:00                  #
 # ================================================== #
 
 from PySide6.QtCore import Qt
@@ -39,6 +39,21 @@ class ChatOutput(QTextBrowser):
         self.anchorClicked.connect(self.open_external_link)
         self.setWordWrapMode(QTextOption.WordWrap)
         self.tab = None
+        self.installEventFilter(self)
+        self.setProperty('class', 'layout-output-plain')
+
+    def eventFilter(self, source, event):
+        """
+        Focus event filter
+
+        :param source: source
+        :param event: event
+        """
+        if event.type() == event.Type.FocusIn:
+            if self.tab is not None:
+                col_idx = self.tab.column_idx
+                self.window.controller.ui.tabs.on_column_focus(col_idx)
+        return super().eventFilter(source, event)
 
     def set_tab(self, tab):
         """
