@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.12 00:00:00                  #
+# Updated Date: 2025.07.22 15:00:00                  #
 # ================================================== #
 
 import re
@@ -48,6 +48,7 @@ class ChatWebOutput(QWebEngineView):
         self.html_content = ""
         self.meta = None
         self.tab = None
+        self.setProperty('class', 'layout-output-web')
 
     def eventFilter(self, source, event):
         """
@@ -165,7 +166,7 @@ class ChatWebOutput(QWebEngineView):
 
         action = QAction(QIcon(":/icons/search.svg"), trans('text.context_menu.find'), self)
         action.triggered.connect(self.find_open)
-        action.setShortcut(QKeySequence("Ctrl+F"))
+        #action.setShortcut(QKeySequence("Ctrl+F"))
         menu.addAction(action)
 
         menu.exec_(self.mapToGlobal(position))
@@ -353,6 +354,16 @@ class Bridge(QObject):
         """
         self.window.controller.ctx.extra.preview_code_text(text)
 
+    @Slot(str)
+    def run_text(self, text: str):
+        """
+        Run code
+
+        :param text: text
+        """
+        self.window.controller.ctx.extra.run_code_text(text)
+
+
     @Slot(int)
     def update_scroll_position(self, pos: int):
         """
